#ifndef __3DSLOADER_H
#define __3DSLOADER_H

#define MAX_3DS_OBJECTS 128 // max number of objects per 3ds file

struct load3ds_vtx_t {
  float x, y, z;
};

struct load3ds_uvvtx_t {
  float u, v;
};

struct load3ds_vertex_t {
  load3ds_vtx_t   vlocal;

  bool            uv_set;
  load3ds_uvvtx_t uv;
};

struct load3ds_face_t {
  int v_index[3];
};

struct load3ds_object_t {
  int nfaces;
  int nvertices;

  load3ds_vertex_t *vertices;
  load3ds_face_t *faces;
};

struct load3ds_chunk_t {
  unsigned short id;
  unsigned int   len;
  unsigned int   dataofs, datalen;

  unsigned char  *data;
  load3ds_chunk_t *child;
  load3ds_chunk_t *next;

  unsigned int   paramsofs;
  unsigned char  *params;
  unsigned int   paramslen;
};

class C3DSLoader {
public:
  C3DSLoader();
  ~C3DSLoader();

  bool Load3DS(char const *filename);

  int m_nObjects;
  load3ds_object_t m_pObjects[MAX_3DS_OBJECTS];

private:
  bool HasName(unsigned short id);
  bool HasChild(unsigned short pid, unsigned short id);

  bool SkipName(load3ds_chunk_t *chunk);

  bool ReadChunk(load3ds_chunk_t *chunk, unsigned int ofs, load3ds_chunk_t *parentchunk = NULL);
  bool ReadChunkTree(load3ds_chunk_t *rootchunk);

  load3ds_chunk_t *FindSubChunk(load3ds_chunk_t *parent, unsigned short id, int cnt = 0);
  
  unsigned short ReadPossibleId(unsigned int ofs);
  unsigned short ReadShort(unsigned int ofs);
  unsigned int   ReadInt(unsigned int ofs);
  float          ReadFloat(unsigned int ofs);

  unsigned char  *m_data;
  unsigned int   m_datalen;
};

#endif