#include "stdafx.h"
#include "bmploader.h"

// ************************* implementation of bitmap loader (BMP only)

class impBmp
{
public:
  impBmp();
  ~impBmp();

  int  GetWidth();
  int  GetHeight();
  int  *GetBits();
  void Resize(int neww, int newh);
  bool LoadFromFile(const char *strFileName);

private:
  int readint(FILE *file);
  int *LoadBMP32(const char *name, int *w, int *h);
  int *m_data;
  int m_w, m_h;
};

impBmp::impBmp()
{
  m_data = NULL;
  m_w = 0;
  m_h = 0;
}

impBmp::~impBmp()
{
  if(m_data != NULL)
  {
    delete []m_data;
    m_data = NULL;
  }
}

int impBmp::GetWidth()
{
  return m_w;
}

int impBmp::GetHeight()
{
  return m_h;
}

int *impBmp::GetBits()
{
  return m_data;
}

void impBmp::Resize(int neww, int newh)
{
  assert(m_data);
  if(m_data == NULL)
    return;

  int *newdata = new int[neww*newh];
  assert(newdata);
  if(newdata == NULL)
    return;

  float dx = (float)m_w/neww;
  float dy = (float)m_h/newh;

  for(int y=0; y<newh; y++)
  {
    for(int x=0; x<neww; x++)
    {
      int x2, y2;
      x2 = (int)(x * dx);
      y2 = (int)(y * dy);

      if(x2 < 0)
        x2 = 0;
      if(x2 > m_w-1)
        x2 = m_w-1;
      if(y2 < 0)
        y2 = 0;
      if(y2 > m_h-1)
        y2 = m_h-1;
      newdata[y*neww+x] = m_data[y2*m_w+x2];
    }
  }

  m_w = neww;
  m_h = newh;

  delete []m_data;
  m_data = newdata;
}

int impBmp::readint(FILE *file)
{
  int a = (unsigned)fgetc(file);
  a += (unsigned)fgetc(file)*0x100;
  a += (unsigned)fgetc(file)*0x10000;
  a += (unsigned)fgetc(file)*0x1000000;
  
  return a;
}

int *impBmp::LoadBMP32(const char *name, int *w, int *h)
{
  int  x, y;
  FILE *file;
  int  *tex;
  char sig[10];
  unsigned char *temp;
  int  overrun=0;

  file = fopen(name, "rb");
  if(file == NULL)
    return 0;

  fread(sig, 2, 1, file);

  if(memcmp(sig, "BM", 2) != 0)
  {
    fclose(file);
    return 0;
  }

  fseek(file, 0x12, 0);
  
  *w = readint(file);
  *h = readint(file);

  overrun = (4-((*w * 3)&3))&3;

  tex = new int[(*w) * (*h)];
  if(tex == NULL)
  {
    fclose(file);
    return 0;
  }

  temp = new unsigned char [(*w+overrun) * (*h) * 3];
  if(temp == NULL)
  {
    fclose(file);
    return 0;
  }

  fseek(file, 54, 0);
  fread(temp, (*w+overrun) * *h * 3, 1, file);

  int p=0;
  for(y=*h-1; y>=0; y--)
  {
    for(x=0; x<*w; x++)
    {
#ifndef SPARC
      tex[y* *w+x] = temp[p] + 256*temp[p+1] + 65536*temp[p+2];
#else
      int r = (int)temp[p+2];
      int g = (int)temp[p+1];
      int b = (int)temp[p+0];

      tex[y* *w+x] = (r << 24) + (b << 8) + (g << 0);
#endif

      p += 3;
    }

    p += overrun;
  }

  delete []temp;

  fclose(file);

  return tex;
}

bool impBmp::LoadFromFile(const char *strFileName)
{
  m_data = LoadBMP32(strFileName, &m_w, &m_h);

  if(m_data == NULL)
    return false;

  return true;
}

// ************************* abstract loader wrapper

BmpLoader::BmpLoader()
{
  m_pBMP = NULL;
}

BmpLoader::~BmpLoader()
{
  delete m_pBMP;
}

int BmpLoader::GetXRes()
{
  if(!m_pBMP)
    return 0;

  return m_pBMP->GetWidth();
}

int BmpLoader::GetYRes()
{
  if(!m_pBMP)
    return 0;

  return m_pBMP->GetHeight();
}

int *BmpLoader::GetRAWptr()
{
  if(!m_pBMP) 
    return NULL;

  return m_pBMP->GetBits();
}

void BmpLoader::Odd2Even()
{
  if((GetXRes()&1) == 0)
    return;

  Resize(GetXRes()+1, GetYRes());
}

void BmpLoader::BGR2RGB(int *data, int w, int h)
{
  for(int i=0; i<w*h; i++)
  {
    int alpha = (data[i]>>24)&255;
    int r     = (data[i]>>16)&255;
    int g     = (data[i]>>8)&255;
    int b     = (data[i]>>0)&255;

    data[i] = (alpha<<24) | (b<<16) | (g<<8) | (r<<0);
  }
}

bool BmpLoader::LoadFromFile(const char *strFileName) 
{
  // check if file exists
  FILE *file = fopen(strFileName, "rb");
  if(file == NULL)
    return false;
  fclose(file);

  delete m_pBMP;
  m_pBMP = new impBmp();

  if(!m_pBMP->LoadFromFile(strFileName))
    return false;

  BGR2RGB(GetRAWptr(), GetXRes(), GetYRes());

  Odd2Even(); // make sure width is even

  return true;
}

void BmpLoader::Resize(int x, int y)
{
  m_pBMP->Resize(x, y);
}

// rounds w to 'nearest' power of 2
int BmpLoader::RoundPower2(int w)
{
  int neww = 1;

  while(neww < w)
    neww *= 2;

  if(w*w < neww * neww / 2)  // round to smaller 2^n
    neww /= 2;

  if(neww < MIN_TEX_DIM)
    neww = MIN_TEX_DIM;

  if(neww > MAX_TEX_DIM)
    neww = MAX_TEX_DIM;

  return neww;
}

void BmpLoader::ResizeToTexture()
{
  Resize(RoundPower2(GetXRes()), RoundPower2(GetYRes()));
}