#include "stdafx.h"
 
#include <math.h> 
#include "collisiondedector.h" 
#include "simdef.h" 

#define EMIT_PARTICLE_SPEED 5.0f
 
void CollisionDedector::init(int nofp, GLLIB_vtx* ps, Point* pt) 
{ 
  pts = pt; 
  pos = ps; 
  numOfPoints = nofp;

  particle = NULL;
} 
 
void CollisionDedector::SetParticles(ParticleSim *p)
{
  particle = p;
}

void CollisionDedector::handleCollisions() 
{   
  for(int i=0; i<numOfPoints; i++)  
  { 
    GLLIB_vtx n, p0;
      
    float d = ground.GetDistance(&n, &p0, &pos[i]);
    if(d < 0.0f)
    {
      p0 = SubVector(p0, pos[i]);
      GLLIB_vtx pn = MultiplyVector(n, d);
      pos[i] = SubVector(pos[i], pn);
      pos[i] = SubVector(pos[i], pn);
    
      float vn = pts[i].velocity.x*n.x + pts[i].velocity.y*n.y + pts[i].velocity.z*n.z;
      vn *= 2;
    
      pts[i].velocity.x -= n.x*vn;
      pts[i].velocity.y -= n.y*vn;
      pts[i].velocity.z -= n.z*vn;
    }
  } 
} 

#define MU_G 0.9f
#define MU_H 1.0f
#define EPS 0.0001f

void CollisionDedector::handleFriction()
{  
  for(int i=0; i<numOfPoints; i++)  
  { 
    GLLIB_vtx n, p0;    
      
    float d = ground.GetDistance(&n, &p0, &pos[i]);
      
    if((d - INTERVAL) < 0.0f) {
      float nfy = pts[i].force.x*n.x + pts[i].force.y*n.y + pts[i].force.z*n.z;
  
      if(nfy < 0.0f) {
        float nvy = pts[i].velocity.x*n.x + pts[i].velocity.y*n.y + pts[i].velocity.z*n.z;
    
        GLLIB_vtx pvy;
        pvy.x = pts[i].velocity.x - nvy*n.x;
        pvy.y = pts[i].velocity.y - nvy*n.y;
        pvy.z = pts[i].velocity.z - nvy*n.z;
    
        float l = VectorLength(pvy);
    
        if(l > EMIT_PARTICLE_SPEED)
        {
          GLLIB_vtx v0 = {pts[i].velocity.x*0.5f, pts[i].velocity.y*0.5f, pts[i].velocity.z*0.5f};
          particle->NewParticle(pos[i], v0);
        }
    
        float gf = -MU_G*nfy;
        float hf = -MU_H*nfy;
    
        GLLIB_vtx pfy;
        pfy.x = pts[i].force.x - nfy*n.x;
        pfy.y = pts[i].force.y - nfy*n.y;
        pfy.z = pts[i].force.z - nfy*n.z;
    
        float f = VectorLength(pfy);
    
        if( ((l < EPS) && (hf > f)) ||  
        (((gf-f)*pts[i].mass_r*STEP_SIZE) > l) ) { 
          // HaftReibung
          pts[i].force.x -= pfy.x;
          pts[i].force.y -= pfy.y;
          pts[i].force.z -= pfy.z;
      
          if(l > EPS) {
            pvy.x *= 0.5f;
            pvy.y *= 0.5f;
            pvy.z *= 0.5f;
        
            pts[i].velocity.x = pvy.x + n.x*nvy;
            pts[i].velocity.y = pvy.y + n.y*nvy;
            pts[i].velocity.z = pvy.z + n.z*nvy;
          }
      
        } else
        {
          // Gleitreibung    
          pfy.x -= pvy.x*gf/l;
          pfy.y -= pvy.y*gf/l;
          pfy.z -= pvy.z*gf/l;
        
          pts[i].force.x = pfy.x + n.x*nfy;
          pts[i].force.y = pfy.y + n.y*nfy;
          pts[i].force.z = pfy.z + n.z*nfy;
        }
      }
    }
  }
}

Ground *CollisionDedector::getGround()
{
  return &ground;
}

Ground::Ground()
{
  int i,j;
  
#define D 3

  for(i=0; i<GROUND_W+1; i++)
  {
    for(j=0; j<GROUND_W+1; j++)
    {
      hfield[i][j] = 0.5f*((float)cos(i*1.6f)*0.3f + (float)cos(j*1.7f)*0.5f);
      hfield[i][j] *= (1 + 3*((GROUND_W-1-i)*(GROUND_W-1-i)+j*j)/(GROUND_W*GROUND_W));
          
      float u = ((float)i-GROUND_W/2) / (GROUND_W/2);
      float v = ((float)j-GROUND_W/2) / (GROUND_W/2);
    
      hfield[i][j] += 1/(((u-0.7f)*(u-0.7f) + (v-0.3f)*(v-0.3f))*10+0.2f);
    
      hfield[i][j] += 1/(((u-0.5f)*(u-0.5f) + (v-0.6f)*(v-0.6f))*10+0.2f);
    
      hfield[i][j] += 1/(((u-0.6f)*(u-0.6f) + (v+0.4f)*(v+0.4f))*10+0.2f);

      if(i >= GROUND_W/2-D && i <= GROUND_W/2+D
         /*j >= GROUND_W/2-D && j <= GROUND_W/2+D*/)
        hfield[i][j] = 0.0f;
    
      if(i >= 20 && i <= 25 && j <= GROUND_W-20)
        hfield[i][j] = ((j-5)%20)*0.5f;
    
      if(j >= GROUND_W-2)
        hfield[i][j] = 5*(float)cos((i-GROUND_W/2)*3.14/GROUND_W);
    }
  }
  
  o.x = -GROUND_W/2*GROUND_CELLW;
  o.y = 0;
  o.z = -GROUND_W/2*GROUND_CELLW;
  
  for(i=0; i<GROUND_W; i++)
  {
    for(j=0; j<GROUND_W; j++)
    {
      GLLIB_vtx p0, p1, p2, p3;
      p0.x = o.x + i*GROUND_CELLW;
      p0.y = hfield[i][j];
      p0.z = o.z + j*GROUND_CELLW;
    
      p1.x = o.x + (i+1)*GROUND_CELLW;
      p1.y = hfield[i+1][j];
      p1.z = o.z + j*GROUND_CELLW;
    
      p2.x = o.x + (i+1)*GROUND_CELLW;
      p2.y = hfield[i+1][j+1];
      p2.z = o.z + (j+1)*GROUND_CELLW;
    
      p3.x = o.x + i*GROUND_CELLW;
      p3.y = hfield[i][j+1];
      p3.z = o.z + (j+1)*GROUND_CELLW;
    
      GLLIB_vtx a = SubVector(p1, p0);
      GLLIB_vtx b = SubVector(p2, p0);
      GLLIB_vtx c = SubVector(p3, p0);
    
      GLLIB_vtx n = CrossProduct(b, a);
      planes[i][j][0].n = NormalizeVector(n);
      planes[i][j][0].k = DotProduct(planes[i][j][0].n, p0);
    
      n = CrossProduct(c, b);
      planes[i][j][1].n = NormalizeVector(n);
      planes[i][j][1].k = DotProduct(planes[i][j][1].n, p0);
    }
  }
}

float Ground::GetDistance(GLLIB_vtx *n, GLLIB_vtx *p0, GLLIB_vtx *p)
{
  int k;
  float u = (p->x - o.x) / GROUND_CELLW;
  float v = (p->z - o.z) / GROUND_CELLW;
  int i = (int)(u);
  int j = (int)(v);
  
  if(u < 0 || v < 0 || i < 0 || j < 0 || i >= GROUND_W || j >= GROUND_W)
    return 1000000.0f;
  
  u -= i;
  v -= j;
  
  if(u > v)
    k = 0;
  else
    k = 1;
  
  *n = planes[i][j][k].n;
  float kk = planes[i][j][k].k;
  
  float d = p->x*n->x + p->y*n->y + p->z*n->z - kk;
  
  if(d > INTERVAL)
    return d;
  
  p0->x = p->x - n->x*d;
  p0->y = p->y - n->y*d;
  p0->z = p->z - n->z*d;
  
  return d;
}

GLLIB_vtx Ground::GetVtx(int i, int j)
{
  GLLIB_vtx v;
  
  v.x = o.x + i*GROUND_CELLW;
  v.y = hfield[i][j];
  v.z = o.z + j*GROUND_CELLW;
  
  return v;
}
