#include "stdafx.h"

#include "damper.h"
#include <math.h>

#define DAMP 3.0f

void Damper::init(GLLIB_vtx *po, void *p, int i1, int i2, float ks) 
{
  pos = po;
  pt = p;
  p1 = i1;
  p2 = i2;

  float x = pos[p2].x - pos[p1].x;
  float y = pos[p2].y - pos[p1].y; 
  float z = pos[p2].z - pos[p1].z;

  initiallength = (float)sqrt( x*x + y*y + z*z );
  oldlength = initiallength;

  k = ks;
}

void Damper::calcForce() 
{
  float force;
  
  float x = pos[p2].x - pos[p1].x;
  float y = pos[p2].y - pos[p1].y; 
  float z = pos[p2].z - pos[p1].z;

  float l = (float)sqrt( x*x + y*y + z*z );

  force = (float)fabs(l - initiallength) * (l - initiallength) * k;
  
  float damp = -DAMP*(oldlength - l) * STEP_SIZE_R;       
  oldlength = l;
  force = force+damp;
  
  if(l < INTERVAL) force = 0.0f;
  else force = force/l; 
  x = x * force;
  y = y * force;
  z = z * force;
    
  Point *pts = (Point*) pt;
    
  pts[p1].force.x = pts[p1].force.x + x;
  pts[p1].force.y = pts[p1].force.y + y;
  pts[p1].force.z = pts[p1].force.z + z;
  
  pts[p2].force.x = pts[p2].force.x - x;
  pts[p2].force.y = pts[p2].force.y - y;
  pts[p2].force.z = pts[p2].force.z - z;
}