#ifndef __GLLIB_H
#define __GLLIB_H

#ifdef WIN32
  #include <gl/gl.h>
  #include <gl/glu.h>
#else
  #include "GL/gl.h"
  #include "GL/glu.h"
#endif

#include "glextensions.h"
#include "math3d.h"

#define GLLIB_NEAR_Z_CLIP 5
#define GLLIB_FAR_Z_CLIP  5000

#define MAX_TEXTURE_STAGES 2

typedef int GLLIB_COLOR;

// the OpenGL-library

class GLLib {
public:
  GLLib();
  ~GLLib();

  bool SetWindowSize(int w, int h);  
  bool StartRender(GLLIB_COLOR background_color, bool clear_screen = true);
  bool EndRender();

  bool SetCamera(GLLIB_vtx &pos, GLLIB_vtx &lookat, GLLIB_vtx &up);
  bool RenderFace(GLLIB_vtx *vtxlist, GLLIB_uvvtx *uv, GLLIB_vtx *normallist);

  bool RenderFaces(GLLIB_vtx *vtxlist, GLLIB_uvvtx *uv, GLLIB_vtx *normallist, int *facelist, int nfaces);
  bool RenderFacesWireframe(GLLIB_vtx *vtxlist, int *facelist, int nfaces);

  bool RenderPoint(GLLIB_vtx *vtx, float size);

  bool RenderLine(GLLIB_vtx *v1, GLLIB_vtx *v2);

  bool EnableTransparency(float t);
  bool DisableTransparency();

  bool EnableDepthBuffer();
  bool DisableDepthBuffer();

  unsigned CreateTexture(int *data, int w, int h);
  bool DelTexture(unsigned texid);

  bool SetCurrentTexture(int stage, unsigned texid);
  bool DisableTexture(int stage);

  bool SetMaterial(GLLIB_COLOR ambient, GLLIB_COLOR diffuse, GLLIB_COLOR specular, float shininess);

  bool SetModelMatrix(GLLIB_matrix &matrix);

  int  GetLastError();

private:
  bool GetARBExtensions();
  void InitGL();
  bool CheckForError();

  bool SetLights();

  int  m_lasterror;

  bool m_texstage_enabled[MAX_TEXTURE_STAGES];

  bool m_cam_matrix_pushed;

  float m_transparency;

  //openGL ARB extension function pointers (set by GetARBExtensions)
  PFNGLACTIVETEXTUREARBPROC       glActiveTextureARB;          // multitexturing
  PFNGLCLIENTACTIVETEXTUREARBPROC glClientActiveTextureARB;    // multitexturing
  PFNGLMULTITEXCOORD2FVARBPROC    glMultiTexCoord2fvARB;       // multitexturing
};

#endif
