#ifndef __GLMODEL_H
#define __GLMODEL_H

#include "gllib.h"

// note: keep it like this for direct OpenGL compatibility!
struct GLMODEL_face {
  int p[3]; // vertex indices
};

#define GLMODEL_TEX_UV     1 // use uv coordinates
#define GLMODEL_TEX_NORMAL 2 // use normal vectors

class GLModel {
public:
  GLModel(GLLib *gllib);
  ~GLModel();

  // type = GLMODEL_TEX_XX
  bool SetNVtxFaces(int nvtx, int nfaces, int type);

  // recalculate vertex normals
  bool CalcNormals(); 

  // recalculate size of object
  void GLModel::CalcSize();

  bool Load3DS(const char *filename, int object, float scale);

  bool Render();

  void SetMaterial(GLLIB_COLOR ambient, GLLIB_COLOR diffuse, GLLIB_COLOR specular, float shininess);

  GLLIB_vtx    *m_vtx;
  GLLIB_uvvtx  *m_uv;
  GLLIB_vtx    *m_normal;
  GLMODEL_face *m_faces;
  int           m_nvtx;
  int           m_nfaces;

  GLLIB_matrix m_matrix;

  // calculate by CalcSize();
  GLLIB_vtx m_min_coordinate, m_max_coordinate; // min/max pos of boundingbox
  GLLIB_vtx m_middle_coordinate, m_size;        // average, width/height of boundingbox

  // material parameters
  GLLIB_COLOR m_ambient;
  GLLIB_COLOR m_diffuse;
  GLLIB_COLOR m_specular;
  float       m_shininess;

  int         m_texture;

  bool        m_transparent;
  bool        m_wireframe;

private:
  int         m_type;
  GLLib       *m_gllib;
};

#endif