#include "stdafx.h"
#include "hirestimer.h"

#ifdef WIN32
WindowsHiresTimer::WindowsHiresTimer()
{
  QueryPerformanceFrequency(&m_timer_frequency);

  Reset();
}

void WindowsHiresTimer::Reset()
{
  QueryPerformanceCounter(&m_timer_start);
}

float WindowsHiresTimer::Query()
{
  LARGE_INTEGER t;
  QueryPerformanceCounter(&t);

  return (float)(t.QuadPart - m_timer_start.QuadPart) / (m_timer_frequency.QuadPart);
}
#else
LinuxHiresTimer::LinuxHiresTimer()
{
  Reset();
}

void LinuxHiresTimer::Reset()
{
  struct timezone tz;
  gettimeofday(&start, &tz);
}

float LinuxHiresTimer::Query()
{
  struct timezone tz;
  struct timeval tv;

  gettimeofday(&tv, &tz);
  tv.tv_sec -= start.tv_sec;
  tv.tv_usec -= start.tv_usec;

  return (float)tv.tv_sec + (float)tv.tv_usec/1000000.0f;
}
#endif
