#ifndef __HIRESTIMER_H
#define __HIRESTIMER_H

#ifndef WIN32
#include <sys/time.h>
#endif

// abstract class
class HiresTimer
{
public:
  virtual void Reset() = 0;  // resets the timer
  virtual float Query() = 0; // returns time since Reset in seconds
};

// implementations (windows and linux)
#ifdef WIN32

class WindowsHiresTimer : public HiresTimer
{
public:
  WindowsHiresTimer();

  void Reset();
  float Query();

private:
  LARGE_INTEGER m_timer_frequency;
  LARGE_INTEGER m_timer_start;
};

#else

class LinuxHiresTimer : public HiresTimer
{
public:
  LinuxHiresTimer();

  void Reset();
  float Query();

private:
  struct timeval start;
};

#endif // WIN32

#endif // __HIRESTIMER_H
