#include "stdafx.h"
#include <math.h>

#include "math3d.h"

GLLIB_matrix::GLLIB_matrix()
{
  Identity();
}

void GLLIB_matrix::Identity()
{
  for(int i=0; i<4; i++)
    for(int j=0; j<4; j++)
      m_matrix[i][j] = (i == j);
}

float *GLLIB_matrix::GetPtr()
{
  return (float *)m_matrix;
}

void GLLIB_matrix::SetMatrix(float *m)
{
  memcpy(m_matrix, m, sizeof(float)*4*4);
}

void GLLIB_matrix::Inverse()
{
  float a = m_matrix[0][0];
  float b = m_matrix[0][1];
  float c = m_matrix[0][2];
  float d = m_matrix[1][0];
  float e = m_matrix[1][1];
  float f = m_matrix[1][2];
  float g = m_matrix[2][0];
  float h = m_matrix[2][1];
  float i = m_matrix[2][2];
  float j = m_matrix[3][0];
  float k = m_matrix[3][1];
  float l = m_matrix[3][2];

  float ww = 1.0f / (a*(e*i-f*h)-(b*(d*i-f*g)+c*(e*g-d*h)));

  m_matrix[0][0]=(e*i-f*h)*ww;
  m_matrix[0][1]=(c*h-b*i)*ww;
  m_matrix[0][2]=(b*f-c*e)*ww;

  m_matrix[1][0]=(f*g-d*i)*ww;
  m_matrix[1][1]=(a*i-c*g)*ww;
  m_matrix[1][2]=(c*d-a*f)*ww;

  m_matrix[2][0]=(d*h-e*g)*ww;
  m_matrix[2][1]=(b*g-a*h)*ww;
  m_matrix[2][2]=(a*e-b*d)*ww;

  m_matrix[3][0]=(e*(g*l-i*j)+f*(h*j-g*k)-d*(h*l-i*k))*ww;
  m_matrix[3][1]=(a*(h*l-i*k)+b*(i*j-g*l)+c*(g*k-h*j))*ww;
  m_matrix[3][2]=(b*(d*l-f*j)+c*(e*j-d*k)-a*(e*l-f*k))*ww;
}

void GLLIB_matrix::MultiplyByRight(GLLIB_matrix &m)
{
  float matrix [4][4];

  memcpy(&matrix, &m_matrix, sizeof(float)*4*4);

  for(int i=0; i<4; i++)
  {
    for(int j=0; j<4; j++)
    {
      m_matrix[i] [j] = 0;
      for(int k=0; k<4; k++)
      {
         m_matrix[i] [j] += matrix[k] [j] * m.m_matrix[i] [k];
      }
    }
  }
}

void GLLIB_matrix::MultiplyByLeft(GLLIB_matrix &m)
{
  float matrix [4][4];

  memcpy(&matrix, &m_matrix, sizeof(float)*4*4);

  for(int i=0; i<4; i++)
  {
    for(int j=0; j<4; j++)
    {
      m_matrix[i] [j] = 0;
      for(int k=0; k<4; k++)
      {
         m_matrix[i] [j] += m.m_matrix[k] [j] * matrix[i] [k];
      }
    }
  }
}

void GLLIB_matrix::Translate(GLLIB_vtx &t)
{
  m_matrix[3][0] += t.x;
  m_matrix[3][1] += t.y;
  m_matrix[3][2] += t.z;
}

void GLLIB_matrix::Scale(float s)
{
  m_matrix[0][0] *= s;
  m_matrix[0][1] *= s;
  m_matrix[0][2] *= s;
  m_matrix[1][0] *= s;
  m_matrix[1][1] *= s;
  m_matrix[1][2] *= s;
  m_matrix[2][0] *= s;
  m_matrix[2][1] *= s;
  m_matrix[2][2] *= s;
}

void GLLIB_matrix::Scale(GLLIB_vtx &v)
{
  m_matrix[0][0] *= v.x;
  m_matrix[0][1] *= v.x;
  m_matrix[0][2] *= v.x;
  m_matrix[1][0] *= v.y;
  m_matrix[1][1] *= v.y;
  m_matrix[1][2] *= v.y;
  m_matrix[2][0] *= v.z;
  m_matrix[2][1] *= v.z;
  m_matrix[2][2] *= v.z;
}

void GLLIB_matrix::Rotate(GLLIB_vtx &axispos, GLLIB_vtx &axis, float angle)
{
  GLLIB_vtx xaxis, yaxis, zaxis, up1 = {1,0,0}, up2 = {0,1,0};
  GLLIB_matrix tc, tcinv, rot, t;

  zaxis = axis;

  zaxis = NormalizeVector(zaxis);

  if(fabs(DotProduct(up1, zaxis)) <  fabs(DotProduct(up2, zaxis)))
    xaxis = CrossProduct(up1, zaxis);
  else
    xaxis = CrossProduct(up2, zaxis);
  
  xaxis = NormalizeVector(xaxis);
  yaxis = CrossProduct(zaxis, xaxis);

  tc.m_matrix[0][0] = xaxis.x;
  tc.m_matrix[1][0] = xaxis.y;
  tc.m_matrix[2][0] = xaxis.z;
  tc.m_matrix[3][0] = -DotProduct(xaxis, axispos);

  tc.m_matrix[0][1] = yaxis.x;
  tc.m_matrix[1][1] = yaxis.y;
  tc.m_matrix[2][1] = yaxis.z;
  tc.m_matrix[3][1] = -DotProduct(yaxis, axispos);

  tc.m_matrix[0][2] = zaxis.x;
  tc.m_matrix[1][2] = zaxis.y;
  tc.m_matrix[2][2] = zaxis.z;
  tc.m_matrix[3][2] = -DotProduct(zaxis, axispos);

  tc.m_matrix[0][3] = 0.0f;
  tc.m_matrix[1][3] = 0.0f;
  tc.m_matrix[2][3] = 0.0f;
  tc.m_matrix[3][3] = 1.0f;

  float _sin = (float)sin(angle);
  float _cos = (float)cos(angle);

  rot.m_matrix[0][0] = _cos;
  rot.m_matrix[0][1] = _sin;
  rot.m_matrix[1][0] = -_sin;
  rot.m_matrix[1][1] = _cos;

  //m: Transform world -> rot.axis system.

  tcinv.m_matrix[0][0] = xaxis.x;
  tcinv.m_matrix[0][1] = xaxis.y;
  tcinv.m_matrix[0][2] = xaxis.z;
  tcinv.m_matrix[0][3] = 0;


  tcinv.m_matrix[1][0] = yaxis.x;
  tcinv.m_matrix[1][1] = yaxis.y;
  tcinv.m_matrix[1][2] = yaxis.z;
  tcinv.m_matrix[1][3] = 0;

  tcinv.m_matrix[2][0] = zaxis.x;
  tcinv.m_matrix[2][1] = zaxis.y;
  tcinv.m_matrix[2][2] = zaxis.z;
  tcinv.m_matrix[2][3] = 0;

  tcinv.m_matrix[3][0] = axispos.x;
  tcinv.m_matrix[3][1] = axispos.y;
  tcinv.m_matrix[3][2] = axispos.z;
  tcinv.m_matrix[3][3] = 1.0f;

  // tcinv * rot * tc * oldm

  MultiplyByLeft(tc);
  MultiplyByLeft(rot);
  MultiplyByLeft(tcinv);
}

GLLIB_vtx GLLIB_matrix::ApplyToVtx(GLLIB_vtx &v)
{
  GLLIB_vtx t;
  t.x = v.x*m_matrix[0][0] + v.y*m_matrix[1][0] +
        v.z*m_matrix[2][0] + m_matrix[3][0];
  t.y = v.x*m_matrix[0][1] + v.y*m_matrix[1][1] +
        v.z*m_matrix[2][1] + m_matrix[3][1];
  t.z = v.x*m_matrix[0][2] + v.y*m_matrix[1][2] +
        v.z*m_matrix[2][2] + m_matrix[3][2];

  return t;
}

GLLIB_vtx GLLIB_matrix::ApplyToVector(GLLIB_vtx &v)
{
  GLLIB_vtx t;
  t.x = v.x*m_matrix[0][0] + v.y*m_matrix[1][0] +
        v.z*m_matrix[2][0];
  t.y = v.x*m_matrix[0][1] + v.y*m_matrix[1][1] +
        v.z*m_matrix[2][1];
  t.z = v.x*m_matrix[0][2] + v.y*m_matrix[1][2] +
        v.z*m_matrix[2][2];

  // don't apply translation

  return t;
}

// global vector stuff

GLLIB_vtx AddVector(GLLIB_vtx &a, GLLIB_vtx &b)
{
  GLLIB_vtx v;
  v.x = a.x + b.x;
  v.y = a.y + b.y;
  v.z = a.z + b.z;

  return v;
}

GLLIB_vtx SubVector(GLLIB_vtx &a, GLLIB_vtx &b)
{
  GLLIB_vtx v;
  v.x = a.x - b.x;
  v.y = a.y - b.y;
  v.z = a.z - b.z;

  return v;
}

GLLIB_vtx MultiplyVector(GLLIB_vtx &v, float a)
{
  GLLIB_vtx r;
  r.x = v.x*a;
  r.y = v.y*a;
  r.z = v.z*a;

  return r;
}

GLLIB_vtx InitVector()
{
  GLLIB_vtx v = {0,0,0};

  return v;
}

float DotProduct(GLLIB_vtx &a, GLLIB_vtx &b)
{
  return a.x*b.x + a.y*b.y + a.z*b.z;
}

GLLIB_vtx CrossProduct(GLLIB_vtx &a, GLLIB_vtx &b)
{
  GLLIB_vtx n;

  n.x = a.y*b.z - a.z*b.y;
  n.y = a.z*b.x - a.x*b.z;
  n.z = a.x*b.y - a.y*b.x;

  return n;
}

GLLIB_vtx NormalizeVector(GLLIB_vtx &v)
{
  GLLIB_vtx n = v;
  float l = VectorLength(v);

  n.x /= l;
  n.y /= l;
  n.z /= l;

  return n;
}