#ifndef __MATH3D_H
#define __MATH3D_H

#include <math.h>

// primitive structures

struct GLLIB_vtx {
  float x, y, z;
};

struct GLLIB_uvvtx {
  float u, v;
};

/*
  what the transformation matrix of GLLIB_matrix looks like:

  matrix = [ m[0][0] m[1][0] m[2][0] m[3][0] ]
           [ m[0][1] m[1][1] m[2][1] m[3][1] ]
           [ m[0][2] m[1][2] m[2][2] m[3][2] ]
           [ m[0][3] m[1][3] m[2][3] m[3][3] ]

         = [ u.x u.y u.z t.x ]
           [ v.x v.y v.z t.y ]
           [ w.x w.y w.z t.z ]
           [ 0   0   0   1   ]
*/

class GLLIB_matrix {
public:
  GLLIB_matrix();

  float *GetPtr(); // get pointer to matrix data;
  void  SetMatrix(float *m);

  void  Identity();
  void  Inverse();
  void  Translate(GLLIB_vtx &t);
  void  Rotate(GLLIB_vtx &axispos, GLLIB_vtx &axis, float angle);
  void  MultiplyByLeft(GLLIB_matrix &m);
  void  MultiplyByRight(GLLIB_matrix &m);

  void  Scale(float s);
  void  Scale(GLLIB_vtx &v);

  GLLIB_vtx ApplyToVtx(GLLIB_vtx &v);
  GLLIB_vtx ApplyToVector(GLLIB_vtx &v);

protected:
  float m_matrix [4][4];
};

GLLIB_vtx AddVector(GLLIB_vtx &a, GLLIB_vtx &b);
GLLIB_vtx SubVector(GLLIB_vtx &a, GLLIB_vtx &b);
GLLIB_vtx MultiplyVector(GLLIB_vtx &v, float a);

GLLIB_vtx InitVector();

float     DotProduct(GLLIB_vtx &a, GLLIB_vtx &b);
GLLIB_vtx CrossProduct(GLLIB_vtx &a, GLLIB_vtx &b);
GLLIB_vtx NormalizeVector(GLLIB_vtx &v);

#define   VectorLength(v) (float)sqrt((v).x*(v).x + (v).y*(v).y + (v).z*(v).z)

#endif
