// -----------------------------------------------------------------------------
// File      : MonsterTruck.cpp
// Component : MonsterTruck main application
// Project   : MonsterTruck
// -----------------------------------------------------------------------------
//         Authors         Email
//         --------------- -----
//         Tobias Gysi     tgysi@student.ethz.ch
//         Peter Kaufmann  peterkau@student.ethz.ch
// -----------------------------------------------------------------------------
//
// NOTE: to compile on SPARC, uncomment '#define SPARC' in stdafx.h
//

#include "stdafx.h"

#ifdef SPARC
#include "SPARC/glut.h"
#else
#include "GL/glut.h"
#endif

#include "gllib.h"
#include "simplemainengine.h"
#include "hirestimer.h"

GLLib      *g_gllib;    // global 3d engine
MainEngine *g_engine;   // global rendering and simulation engine

HiresTimer *g_timer;    // global timer

void callback_display(void)
{
  float t = g_timer->Query();
  g_engine->RenderStep(t);

  glutSwapBuffers();
}

void callback_reshape(int w, int h)
{
  g_gllib->SetWindowSize(w, h);
}

void callback_animate(void)
{
  glutPostRedisplay();
}

void processSpecialKeys(int key, int x, int y) 
{
  switch(key) 
  {
    case GLUT_KEY_UP:
    case GLUT_KEY_DOWN:
    case GLUT_KEY_LEFT:
    case GLUT_KEY_RIGHT:
    case GLUT_KEY_END:
    case GLUT_KEY_F1:
    case GLUT_KEY_F2:
    case GLUT_KEY_F3:
    case GLUT_KEY_F4:
    case GLUT_KEY_F5:
    case GLUT_KEY_F6:
    case GLUT_KEY_F7:
    case GLUT_KEY_F8:
      g_engine->SetKeyState(key + 0x100, true);
      break;
    default:
      break;
  }
}

void processSpecialUpKeys(int key, int x, int y)
{
  switch(key) 
  {
    case GLUT_KEY_UP:
    case GLUT_KEY_DOWN:
    case GLUT_KEY_LEFT:
    case GLUT_KEY_RIGHT:
    case GLUT_KEY_END:
    case GLUT_KEY_F1:
    case GLUT_KEY_F2:
    case GLUT_KEY_F3:
    case GLUT_KEY_F4:
    case GLUT_KEY_F5:
    case GLUT_KEY_F6:
    case GLUT_KEY_F7:
    case GLUT_KEY_F8:
      g_engine->SetKeyState(key + 0x100, false);
      break;
    default:
      break;
  }
}

void processKeys(unsigned char key, int x, int y)
{
  g_engine->SetKeyState(key, true);
}

void processUpKeys(unsigned char key, int x, int y)
{
  g_engine->SetKeyState(key, false);
}

void PrintControls()
{
  printf("              ** MONSTERTRUCK **\n");
  printf("                   2003 by\n");
  printf("      Tobias Gysi (tgysi@student.ethz.ch)\n");
  printf("   Peter Kaufmann (peterkau@student.ethz.ch)\n");
  printf("\n\n");
  printf("cursor left/right: steer left/right\n");
  printf("cursor up/down:    accelerate forward/backwards\n");
  printf("space:             brake\n\n");
  printf("F1-F4:             change camera\n");
  printf("q, e:              look left/right/rear\n\n");
  printf("F5:                toggle rendering car model\n");
  printf("F6:                toggle rendering mass/spring model\n");
  printf("F7:                toggle rendering particles\n");
  printf("F8:                toggle rendering floor wireframe\n\n");
  printf("ESC:               reset simulation\n");
}

int main(int argc, char* argv[])
{
  // init GLUT
  glutInit(&argc, argv);

  glutInitWindowSize(640, 480);
  glutInitDisplayMode(GLUT_DOUBLE|GLUT_RGB|GLUT_SINGLE|GLUT_DEPTH);
  glutCreateWindow("MonsterTruck");

  // init 3D-engine
  g_gllib = new GLLib();
    
  // enit rendering engine
  g_engine = new SimpleMainEngine(); // using Simple engine here
  g_engine->Set3D(g_gllib);
  g_engine->InitAll();

#ifdef WIN32
  g_timer = new WindowsHiresTimer;
#else
  g_timer = new LinuxHiresTimer;
#endif

  // set callbacks
  glutDisplayFunc(callback_display);
  glutReshapeFunc(callback_reshape);
  glutIdleFunc(callback_animate);

  glutSpecialFunc(processSpecialKeys);
  glutSpecialUpFunc(processSpecialUpKeys);
  glutKeyboardFunc(processKeys);
  glutKeyboardUpFunc(processUpKeys);

  // print control keys
  PrintControls();

  // main loop
  glFlush();

  g_timer->Reset();
  glutMainLoop(); 
  
  // free everything
  if(g_engine != NULL)
  {
    g_engine->DeinitAll();
    delete g_engine;
    g_engine = NULL;
  }

  if(g_gllib != NULL)
  {
    delete g_gllib;
    g_gllib = NULL;
  }

  return 0;
}