#include "stdafx.h"
#include "particlesim.h"
#include "collisiondedector.h"

ParticleSim::ParticleSim(int max_particles, Ground *ground)
{
  m_p = NULL;
  m_n = max_particles;

  m_p = new particle_t[m_n];

  for(int i=0; i<m_n; i++)
    m_p[i].alive = false;

  m_idx_alivestart = 0;
  m_nalive = 0;

  m_ground = ground;

  m_time = 0.0f;
  m_deltatime = 0.0f;
}

void ParticleSim::Simulate(float time, float deltat)
{
  GLLIB_vtx n, p0;
  int new_alivestart = m_idx_alivestart;
  int new_nalive = m_nalive;

  for(int j=0; j<m_nalive; j++)
  {
    int i = (j+m_idx_alivestart)%m_n;

    if(m_p[i].alive)
    {
      m_p[i].p.x += m_p[i].v.x * deltat;
      m_p[i].p.y += m_p[i].v.y * deltat;
      m_p[i].p.z += m_p[i].v.z * deltat;

      m_p[i].v.x *= PARTICLE_DRAG;
      m_p[i].v.y *= PARTICLE_DRAG;
      m_p[i].v.z *= PARTICLE_DRAG;

      m_p[i].v.y += PARTICLE_G*deltat;

      float d = m_ground->GetDistance(&n, &p0, &m_p[i].p);
      if(d < 0)
      {
        float dd = DotProduct(n, m_p[i].v);
        m_p[i].v.x -= n.x*dd;
        m_p[i].v.y -= n.y*dd;
        m_p[i].v.z -= n.z*dd;

        m_p[i].v.x *= PARTICLE_FRICTION;
        m_p[i].v.y *= PARTICLE_FRICTION;
        m_p[i].v.z *= PARTICLE_FRICTION;

        m_p[i].v.x -= n.x*dd*0.13f;
        m_p[i].v.y -= n.y*dd*0.13f;
        m_p[i].v.z -= n.z*dd*0.13f;

        m_p[i].p.x -= n.x*d*2;
        m_p[i].p.y -= n.y*d*2;
        m_p[i].p.z -= n.z*d*2;

        //m_p[i].alive = false;
      }

      if(time - m_p[i].life_start > MAX_PARTICLE_LIFETIME)
      {
        // particle too old -> alle lower particles are too old, too
        new_alivestart = (i+1)%m_n;
        new_nalive--;
      }
    }
  }

  m_idx_alivestart = new_alivestart;
  m_nalive = new_nalive;

  assert(m_nalive >= 0);
}

void ParticleSim::SetTime(float time)
{
  m_deltatime = time - m_time;
  m_time = time;
}

void ParticleSim::NewParticle(GLLIB_vtx &pos, GLLIB_vtx &v0)
{
  particle_t *p;

  float r = (rand()%10000)/10000.0f;  // [0,1]

  if(r >= m_deltatime*PARTICLE_EMITRATE)
    return;

  if(m_nalive < m_n)
  {
    p = &m_p[(m_idx_alivestart + m_nalive) % m_n];
    m_nalive++;
  }
  else
  {
    // m_nalive == m_n -> delete oldest particle
    p = &m_p[m_idx_alivestart];
    m_idx_alivestart= (m_idx_alivestart+1) % m_n;
  }

  int color_r = 100;
  int color_g = 70;
  int color_b = 30;

  int br = rand()%50;
  color_r += br;
  color_g += br;
  color_b += br;

  if(color_r > 255)
    color_r = 255;
  if(color_g > 255)
    color_g = 255;
  if(color_b > 255)
    color_b = 255;

  v0.x += PARTICLE_DIR_VARIANCE * ((rand()%1000) / 1000.0f) - 0.0005f;
  v0.y += PARTICLE_DIR_VARIANCE * ((rand()%1000) / 1000.0f) - 0.0005f;
  v0.z += PARTICLE_DIR_VARIANCE * ((rand()%1000) / 1000.0f) - 0.0005f;

  p->v = v0;
  p->p = pos;
  p->alive = true;
  p->color = (color_r<<16) + (color_g<<8) + color_b;
  p->life_start = m_time;

  p->rndvect.x = (rand()%10000)/10000.0f;
  p->rndvect.y = (rand()%10000)/10000.0f;
  p->rndvect.z = (rand()%10000)/10000.0f;
}

particle_t *ParticleSim::GetParticles(int *totaln, int *start, int *n)
{
  *totaln = m_n;
  *start = m_idx_alivestart;
  *n = m_nalive;

  return m_p;
}

ParticleSim::~ParticleSim()
{
  if(m_p != NULL)
  {
    delete []m_p;
    m_p = NULL;
  }
}
