#include "stdafx.h"
#include "poweredwheel.h"
#include "math3d.h"

#include <math.h>

#define PI 3.1415926f

PoweredWheel::PoweredWheel()
{
  m_pointindex = NULL;
  m_points = NULL;
}

PoweredWheel::~PoweredWheel()
{
  if(m_pointindex)
  {
    delete []m_pointindex;
    m_pointindex = NULL;
  }
}

void PoweredWheel::Init(GLLIB_vtx *pos, Point *points, int npoints, int *wheel_points, int axisa, int axisb)
{
  m_pos = pos;
  
  m_points = points;
  m_npoints = npoints;
  
  m_axisa = axisa;
  m_axisb = axisb;

  m_pointindex = new int[npoints];

  for(int i=0; i<npoints; i++)
    m_pointindex[i] = wheel_points[i];

  torque = 0.0f;
  pressure = 0.0f;
  rotSpeed = 0.0f;
}

void PoweredWheel::SetTorque(float t) {
  torque = t;
}

void PoweredWheel::SetBreak(float p) {
  pressure = p;
}

float PoweredWheel::GetRotSpeed() {
  return rotSpeed;
}

void PoweredWheel::calcSpeed() {
  GLLIB_vtx a = m_pos[m_axisa];
  GLLIB_vtx b = m_pos[m_axisb];
  
  int p1 = m_pointindex[0];
  
  GLLIB_vtx p = m_pos[p1];
  
  GLLIB_vtx A = SubVector(b, a);
  GLLIB_vtx B = SubVector(p, a);
  
  GLLIB_vtx n = CrossProduct(B, A);
  float l = (float)sqrt(n.x * n.x + n.y * n.y + n.z * n.z);
  n.x /= l;
  n.y /= l;
  n.z /= l;
  
  float v = n.x*m_points[p1].velocity.x + n.y*m_points[p1].velocity.y + n.z*m_points[p1].velocity.z; 
  float d = VectorLength(B);
  
  rotSpeed = (v/d) / (2*PI);
}

#define MAX_FORCE 50.0f
#define FORCE_SCALE 5.0f
#define DAMP 1.0f

void PoweredWheel::applyForces() {
  // Bremsen
  if(rotSpeed > 0.0f)
    pressure *= pressure * FORCE_SCALE * -(rotSpeed + 1.0f);
  else 
    pressure *= pressure * FORCE_SCALE * -(rotSpeed - 1.0f);

  if(pressure > MAX_FORCE) {
    pressure = MAX_FORCE;
  }
  if(pressure < -MAX_FORCE) {
    pressure = -MAX_FORCE;
  }

  // Reibung
  float friction = DAMP * (float)fabs(rotSpeed) * -rotSpeed; 

  GLLIB_vtx a = m_pos[m_axisa];
  GLLIB_vtx b = m_pos[m_axisb];
  GLLIB_vtx da = SubVector(a, b);

  for(int i=0; i<m_npoints; i++)
  {
    GLLIB_vtx p = m_pos[m_pointindex[i]];
    GLLIB_vtx db = SubVector(p, b);
    GLLIB_vtx norm = CrossProduct(da, db);
      
    float f = (torque + pressure + friction) / 
              (m_npoints * (float)sqrt(norm.x * norm.x + norm.y * norm.y +  norm.z * norm.z));
    norm.x *= f;
    norm.y *= f;
    norm.z *= f;
            
    m_points[m_pointindex[i]].force.x += norm.x;
    m_points[m_pointindex[i]].force.y += norm.y;
    m_points[m_pointindex[i]].force.z += norm.z;
  }
}
