#include "stdafx.h"
#include "procrustes.h"
#include "svd.h"

void Procrustes::Matrix_Transpose(float *to, float *from, int m, int n)
{
  for(int i=0; i<m; i++)
  {
    for(int j=0; j<n; j++)
    {
      to[i + j*m] = from[j + i*n];
    }
  }
}

void Procrustes::Matrix_Mul(float *dest, float *a, float *b, int ma, int na, int nb)
{
  int mb = na;
  int mc = ma;
  int nc = nb;

  for(int i=0; i<mc; i++)
  {
    for(int j=0; j<nc; j++)
    {
      dest[j + i*nc] = 0;

      for(int k=0; k<na; k++)
      {
        dest[j + i*nc] += a[i*na + k]*b[k*nb + j];
      }
    }
  }
}

float Procrustes::Matrix_Det(float *a, int m, int n)
{
  float d = 0;

  assert(m == 3 && n == 3);

  if(m != 3 || n != 3)
    return 0;

  d += a[0]*(a[4]*a[8] - a[5]*a[7]);
  d -= a[1]*(a[3]*a[8] - a[5]*a[6]);
  d += a[2]*(a[3]*a[7] - a[4]*a[6]);

  return d;
}

void Procrustes::FitObject(GLLIB_matrix &fitmatrix, GLLIB_vtx *to_vtx, GLLIB_vtx *from_vtx, int n)
{
  int i;

  float from[3*MAX_PROCRUSTES_N];
  float to[3*MAX_PROCRUSTES_N];

  GLLIB_vtx origin = from_vtx[0];
  GLLIB_vtx toorigin = to_vtx[0];

  from[0*n + 0] = 0;
  from[1*n + 0] = 0;
  from[2*n + 0] = 0;

  for(i=1; i<n; i++)
  {
    from[0*n + i] = from_vtx[i].x - origin.x;
    from[1*n + i] = from_vtx[i].y - origin.y;
    from[2*n + i] = from_vtx[i].z - origin.z;
  }

  to[0*n + 0] = 0;
  to[1*n + 0] = 0;
  to[2*n + 0] = 0;

  for(i=1; i<n; i++)
  {
    to[0*n + i] = to_vtx[i].x - toorigin.x;
    to[1*n + i] = to_vtx[i].y - toorigin.y;
    to[2*n + i] = to_vtx[i].z - toorigin.z;
  }

  float bT[MAX_PROCRUSTES_N * 3];
  float MbT[3][3];

  Matrix_Transpose(bT, from, 3, n);
  Matrix_Mul((float *)MbT, to, bT, 3, n, 3);

  float u[3][3], v[3][3];
  float w[3];
  float *U[3], *V[3];

  U[0] = &u[0][0];
  U[1] = &u[1][0];
  U[2] = &u[2][0];

  V[0] = &v[0][0];
  V[1] = &v[1][0];
  V[2] = &v[2][0];

  u[0][0] = MbT[0][0];
  u[0][1] = MbT[0][1];
  u[0][2] = MbT[0][2];

  u[1][0] = MbT[1][0];
  u[1][1] = MbT[1][1];
  u[1][2] = MbT[1][2];
  u[2][0] = MbT[2][0];
  u[2][1] = MbT[2][1];
  u[2][2] = MbT[2][2];

  int ret = dsvd(U, 3, 3, w, V);
  assert(ret);

  float vT[3][3];
  Matrix_Transpose((float *)vT, (float *)v, 3, 3);

  float Q[3][3];
  Matrix_Mul((float *)Q, (float *)u, (float *)vT, 3, 3, 3);

  float detQ = Matrix_Det((float*)Q, 3, 3);

  float fixdet[3][3];
  fixdet[0][0] = 1;
  fixdet[0][1] = 0;
  fixdet[0][2] = 0;
  fixdet[1][0] = 0;
  fixdet[1][1] = 1;
  fixdet[1][2] = 0;
  fixdet[2][0] = 0;
  fixdet[2][1] = 0;
  fixdet[2][2] = detQ;

  float newu[3][3];
  Matrix_Mul((float *)newu, (float *)u, (float *)fixdet, 3, 3, 3);

  //fix determinant of Q:
  Matrix_Mul((float *)Q, (float *)newu, (float *)vT, 3, 3, 3);
    
  GLLIB_vtx nullvector = {0,0,0};
  GLLIB_matrix procrustes;

  float procm[4][4];
  procm[0][0] = Q[0][0];
  procm[1][0] = Q[0][1];
  procm[2][0] = Q[0][2];
  procm[3][0] = 0;

  procm[0][1] = Q[1][0];
  procm[1][1] = Q[1][1];
  procm[2][1] = Q[1][2];
  procm[3][1] = 0;

  procm[0][2] = Q[2][0];
  procm[1][2] = Q[2][1];
  procm[2][2] = Q[2][2];
  procm[3][2] = 0;
    
  procm[0][3] = 0;
  procm[1][3] = 0;
  procm[2][3] = 0;
  procm[3][3] = 1;
    
  procrustes.SetMatrix((float *)procm);

  fitmatrix.Identity();
  GLLIB_vtx buf = SubVector(nullvector, origin);
  fitmatrix.Translate(buf);
  fitmatrix.MultiplyByLeft(procrustes);

  fitmatrix.Translate(toorigin);
}