#ifndef __SIMPLEMAINENGINE_H
#define __SIMPLEMAINENGINE_H

#include "mainengine.h"
#include "glmodel.h"
#include "springsim.h"
#include "delayvariable.h" 
#include "particlesim.h" 
#include "vtxinterpolator.h"
#include "engine.h"
 
#define CAM_FIXED  0 
#define CAM_STEER  1 
#define CAM_MOVIE  2 
#define CAM_FOLLOW 3 
 
#define LOOKAT_FRONT 0 
#define LOOKAT_LEFT  1 
#define LOOKAT_BACK  2 
#define LOOKAT_RIGHT 3 

class SimpleMainEngine: public MainEngine
{
public:
  SimpleMainEngine();
  ~SimpleMainEngine();
  bool InitAll();
  bool RenderStep(float time);

  void SetKeyState(int key, bool state);

private: 
  GLLIB_matrix FitCar(); 
  GLLIB_matrix FitWheel(int wheelindex); 
  void RenderSpringModel(); 
  void RenderSkyBox(GLLIB_vtx campos); 
  void RenderParticles(float time); 
  void CalculateStep(float time); 
  GLLIB_vtx SetCamera(float time); 
  bool ResetSim(); 
 
  void ThickLine(GLLIB_vtx v1, GLLIB_vtx v2, float d); 

  unsigned  m_floortex, m_metaltex, m_leathertex; 
  unsigned  m_skytex[6];
  GLModel   *m_carmodel, *m_wheelmodel, *m_wheelinside, *m_floormodel, *m_particlemodel;
  GLModel   *m_car_metal, *m_car_seats, *m_car_steer, *m_car_window;
  SpringSim *m_sim; 
 
  ParticleSim *m_particle;

  bool m_key_state[ENGINE_MAX_KEYS];
  
  DelayVariable m_steervar;
  DelayVariable m_throttlevar; 
  DelayVariable m_breakvar; 

  Engine m_eng;
 
  int   m_cam;        // current camera (CAM_XXX)
  int   m_lookat_dir; // LOOKAT_XXX
 
  float m_simulated_time; // simulated time in seconds
 
  VtxInterpolator m_campos_interp, m_campos_rel_interp; 
  VtxInterpolator m_camlookat_interp, m_camlookat_rel_interp; 
  VtxInterpolator m_camup_interp, m_camup_rel_interp; 
 
  // display flags
  bool m_rendercar; 
  bool m_rendersprings; 
  bool m_renderfloor; 
  bool m_renderparticles;
};

#endif
