#include "stdafx.h"

#include "springsim.h"
#include <fstream.h>

SpringSim::SpringSim() {
  numOfPoints = 0;
  numOfSprings = 0;

  pos = NULL;
  pts = NULL;
  sps = NULL;
}

SpringSim::~SpringSim() {
  if(pts != NULL) {
    delete [] pts; 
  } 
  if(pos != NULL) { 
    delete [] pos; 
  } 
  if(sps != NULL) { 
    delete [] sps; 
  }  
  if(dmp != NULL) { 
    delete [] dmp; 
  }  
  if(ra != NULL) { 
    delete [] ra; 
  } 
  if(sa != NULL) { 
    delete [] sa; 
  } 
  if(whl != NULL) { 
    delete [] whl; 
  } 
} 
 
bool SpringSim::init(char *fname) { 
  int i; 
  ifstream file; 
  file.open(fname); 
   
  if(!file.good()) 
    return false; 
   
  file >> numOfPoints; 
 
  pts = new Point[numOfPoints]; 
  pos = new GLLIB_vtx[numOfPoints]; 
     
  for(i=0; i<numOfPoints; i++)  
  { 
    float x,y,z,m; 
 
    file >> x; file >> y; file >> z; 
    file >> m; 
       
    pts[i].init(i, pos, x, y, z, m); 
  } 

  file >>  numOfSprings; 
  sps = new Spring[numOfSprings]; 
 
  for(i=0; i<numOfSprings; i++)  
  { 
    int a,b; 
    float k,d; 
 
    file >> a; file >> b; 
    file >> k; file >> d; 
       
    sps[i].init(pos, pts, a, b, k, d);  
  } 

  file >>  numOfWheels; 
  whl = new PoweredWheel[numOfWheels];
 
  for(i=0; i<numOfWheels; i++)  
  { 
    int a,b; 
    int wheelpts;
    
    file >> a; file >> b; file >> wheelpts;

    if(wheelpts > 0)
    {
      int *warray = new int[wheelpts];
  
      for(int k=0; k<wheelpts; k++)
      {
        file >> warray[k];
      }
  
      whl[i].Init(pos, pts, wheelpts, warray, a, b);
  
      delete []warray;
    }
  }

  file >>  numOfRAngles; 
  ra = new rightAngle[numOfRAngles];

  for(i=0; i<numOfRAngles; i++)  
  {
    int a,b,c;
    
    file >> a;  file >> b;  file >> c;  
    
    ra[i].init(pos, pts, a, b, c);
  }
  
  cd.init(numOfPoints, pos, pts); 


  file >>  numOfSAngles;
 
  sa = new steeringAngle[numOfSAngles];
  
  for(i=0; i<numOfSAngles; i++) {  
    int p1,p2,p3;
    file >> p1; file >> p2; file >> p3;   
    sa[i].init(pos, pts, p1, p2, p3);
  }
  
  file >>  numOfDampers; 
  dmp = new Damper[numOfDampers]; 
 
  for(i=0; i<numOfDampers; i++)  
  { 
    int a,b; 
    float k; 
 
    file >> a; file >> b; 
    file >> k;  
       
    dmp[i].init(pos, pts, a, b, k);  
  }
 
  return true; 
} 
 
GLLIB_vtx *SpringSim::getPointAddress() { 
  return pos; 
} 

steeringAngle *SpringSim::getSteeringAngleAddress() { 
  return sa; 
} 
 
int SpringSim::getNumOfSprings() { 
  return numOfSprings; 
} 
 
int SpringSim::getNumOfSteeringAngles() { 
  return numOfSAngles; 
} 
 
Spring* SpringSim::getSpringAddress() { 
  return sps; 
} 
 
int SpringSim::getNumOfPts() { 
  return numOfPoints; 
} 

PoweredWheel* SpringSim::getWheelAddress() {
  return whl;
}

int SpringSim::getNumOfWheels() {
  return numOfWheels;
}
 
float SpringSim::step(int msecs) {
  int i; 
  int it = (int)(msecs / (STEP_SIZE*1000)); 

  for(int t=0; t<it; t++) {     
    for(i=0; i<numOfPoints; i++) { 
      pts[i].calcPosition();
    } 
    cd.handleCollisions();

    for(i=0; i<numOfSprings; i++) { 
      sps[i].calcForce(); 
    }

    for(i=0; i<numOfDampers; i++) { 
      dmp[i].calcForce(); 
    }

    cd.handleFriction();
    
    for(i=0; i<numOfSAngles; i++) { 
      sa[i].applyForces();
    }

    for(i=0; i<numOfRAngles; i++) { 
      ra[i].applyForces();
    }
    
    for(i=0; i<numOfWheels; i++) {
      whl[i].applyForces();
    }
    
    for(i=0; i<numOfPoints; i++) {  
      pts[i].calcVelocity(); 
    }
    
    for(i=0; i<numOfWheels; i++) {
      whl[i].calcSpeed();
    }
   
    for(i=0; i<numOfWheels; i++) {
      whl[i].applyForces();
    }
  }
  
  return it * STEP_SIZE * 1000; // milliseconds simulated
}