#include "stdafx.h"
#include "vtxinterpolator.h"

VtxInterpolator::VtxInterpolator()
{
  m_historylen = 0;
  m_historystart = 0;
}

void VtxInterpolator::Set(GLLIB_vtx &v, float t)
{
  if(m_historylen < MAX_VTXHISTORYLEN)
  {
    m_history[(m_historystart+m_historylen) % MAX_VTXHISTORYLEN].v = v;
    m_history[(m_historystart+m_historylen) % MAX_VTXHISTORYLEN].t = t;
    m_historylen++;
  }
  else
  {
    m_history[m_historystart].v = v;
    m_history[m_historystart].t = t;
    m_historystart = (m_historystart+1) % MAX_VTXHISTORYLEN;
  }
}

GLLIB_vtx VtxInterpolator::Get(float t, float deltat)
{
  GLLIB_vtx pos = InitVector();
  float l=0;

  assert(m_historylen > 0);

  if(m_historylen <= 0)
    return pos;

  if(m_historylen == 1)
    return m_history[m_historystart].v;

  for(int i=m_historylen-1; i>=0; i--)
  {
    int j = (i+m_historystart) % MAX_VTXHISTORYLEN;
    int jnext = (j+1) % MAX_VTXHISTORYLEN;

    float frametime;
    if(i < m_historylen-1)
      frametime = m_history[jnext].t - m_history[j].t;
    else
      frametime = t - m_history[j].t;

    if(m_history[j].t < t-deltat)
    {
      frametime -= (t-deltat) - m_history[j].t;

      GLLIB_vtx p = MultiplyVector(m_history[j].v, frametime);
      pos = AddVector(pos, p);

      l += frametime;

      break;
    }

    GLLIB_vtx p = MultiplyVector(m_history[j].v, frametime);
    pos = AddVector(pos, p);

    l += frametime;
  }

  if(l <= 0)
    return m_history[(m_historystart+m_historylen-1) % MAX_VTXHISTORYLEN].v;

  pos = MultiplyVector(pos, 1.0f/l);

  return pos;
}

void VtxInterpolator::Transform(GLLIB_matrix &m)
{
  for(int i=0; i<m_historylen; i++)
  {
    int j = (i+m_historystart) % MAX_VTXHISTORYLEN;

    m_history[j].v = m.ApplyToVector(m_history[j].v);
  }
}